#-------------------------------------------------------------------------------
# Name:        ParameterEdit7
# Purpose:
#
# Author:      Kelvin
#
# Created:     4/12/2013
#-------------------------------------------------------------------------------

# Note to self: Make this program more modular and readable by splitting it into distinct parts and use perhaps use functions to implement them.
# Note to self: Python can also be made into multi-file programs, so that it looks neater. Use 'import' to achieve that.


# Define the file names involved.
defaultParamFileName = '/u/hsu004/IntroToDuchamp/duchampDefaultParameters'
userSpecifiedFileName = '/u/hsu004/IntroToDuchamp/duchampHIPASS.in'

newFileDirectory = '/u/hsu004/Kelvin/'
newFileName = 'duchampHIPASSedited.in'
newFilePath = newFileDirectory + newFileName


# Define a simple function to determine the user's intent.
# Note that this may be very inefficient.
YES = 1
NO = 0
INVALID = -1
def toEdit(response):
    if (response == 'y') | (response == 'Y') | (response == 'yes'):
        return(YES)
    elif (response == 'n') | (response == 'N') | (response == 'no'):
        return(NO)
    else:
        return(INVALID)

    
# Define a function to read parameter files with the specified format.
def readParamFile(paramFileName):

    # Initialise an empty Dictionary.
    parameters = {}

    # Open the given parameter file.
    paramFile = open(paramFileName)

    # Load files for lines of parameter data in the specified format and
    # store them into the dictionary.
    for line in paramFile:
        if line[0] != '#' and len(line) > 1:
            linebits = line.split()
            parameters[linebits[0]] = linebits[1]

    # Go back to the start of the file.
    paramFile.seek(0, 0)

    paramFile.close()

    return parameters

# Define a function to create a parameter file with the specified format.
def writeParamFile(newParamFileName, parameters):

    # Create a new file.
    newParamFile = open(newParamFileName, 'w')

    # Write the editted set of parameters into the new file.
    for par in parameters:
        newParamFile.write('%s    %s\n'%(par, parameters[par]))

    newParamFile.close();

# Define a function to print a list of parameters listed in a dictionary.
def listParameters(parameters):

    for par in parameters:
        print(par, parameters[par])
        
    print('\n')

# Firstly, determine the default list of parameters
defaultParameters = readParamFile(defaultParamFileName)   

# Then, determine the user specified list of parameters
userSpecifiedParameters = readParamFile(userSpecifiedFileName)
        
#-------------------------------------------------------------------------------
# GUI - Tkinter section
from Tkinter import *
  
class mainInterface:
  
    def __init__(self, master):

        self.frame = Frame(master)
        
        self.frame.grid()

        self.parameterLabel = Label(self.frame, text = "Parameter: ", foreground = "Blue")
        self.parameterLabel.grid(row = 1, column = 1, columnspan = 2)

        self.valueLabel = Label(self.frame, text = "Value: ", foreground = "Green")
        self.valueLabel.grid(row = 1, column = 3, columnspan = 2)

        self.parameterEntry = Entry(self.frame, width = 25)
        self.parameterEntry.grid(row = 2, column = 1, padx = 10, columnspan = 2)

        self.valueEntry = Entry(self.frame, width = 25)
        self.valueEntry.grid(row = 2, column = 3, padx = 10, columnspan = 2)

        self.checkButton = Button(self.frame, text = "Check", command = self.checkParameter)
        self.checkButton.grid(row = 3, column = 1, padx = 10, pady = 10)

        self.updateButton = Button(self.frame, text = "Update", command = self.updateParameter)
        self.updateButton.grid(row = 3, column = 2, padx = 10, pady = 10)

        self.listDefault = Button(self.frame, text = "List Default Parameters", command = self.listDefaultParameters)
        self.listDefault.grid(row = 4, column = 1, padx = 10, columnspan = 2)

        self.listSpecified = Button(self.frame, text = "List User Specified Parameters", command = self.listSpecifiedParameters)
        self.listSpecified.grid(row = 5, column = 1, padx = 10, columnspan = 2)

        self.promptSaveButton = Button(self.frame, text = "Save", command = self.promptSaveParameter)
        self.promptSaveButton.grid(row = 4, column  = 4, padx = 20, pady = 10, sticky = E)
        
        self.buttonQUIT = Button(self.frame, text = "QUIT", fg = "red", command = self.frame.quit)
        self.buttonQUIT.grid(row = 5, column = 4, padx = 20, pady = 10, sticky = E)

        

        self.resultTitle = StringVar()
            
        self.result = StringVar()

        self.notesTitle = StringVar()

        self.notes = StringVar()

        
        # New File Name
        self.newParamFileDirectory = StringVar()

        self.newParamFileName = StringVar()

        self.newParamFilePath = StringVar()

    def checkParameter(self):

        parameter2check = self.parameterEntry.get()

        if parameter2check in userSpecifiedParameters:
            value = userSpecifiedParameters[parameter2check]
            paramInfo = parameter2check + ': ' + value
            color = "Green"
            defaultYN = "No"
        elif parameter2check in defaultParameters:
            value = defaultParameters[parameter2check]
            paramInfo = parameter2check + ': ' + value
            color = "Green"
            defaultYN = "Yes"
        else:
            value = "N/A"
            paramInfo = '"' + parameter2check + '" does not exist.'
            color = "Red"
            defaultYN = "N/A"

        self.resultTitle.set("Parameter Information")
        self.result.set(paramInfo)
        self.notesTitle.set("Default Parameter?")
        self.notes.set(defaultYN)
        
        self.resultLabel = Label(self.frame, textvariable = self.resultTitle)
        self.resultLabel.grid(row = 1, column = 5, padx = 10, columnspan = 2)
        
        self.resultValue = Label(self.frame, textvariable = self.result, foreground = color)
        self.resultValue.grid(row = 2, column = 5, padx = 10, columnspan = 2)

        self.valueEntry.delete(0, END)
        self.valueEntry.insert(0, value)

        self.notesLabel = Label(self.frame, textvariable = self.notesTitle)
        self.notesLabel.grid(row = 1, column = 7, padx = 10)

        self.notesValue = Label(self.frame, textvariable = self.notes)
        self.notesValue.grid(row = 2, column = 7, padx = 10)

    def updateParameter(self):

        parameter2update = self.parameterEntry.get()
       
        if parameter2update in userSpecifiedParameters:
            value2write = self.valueEntry.get()
            userSpecifiedParameters[parameter2update] = value2write
            paramInfo = parameter2update + ': ' + value2write
            newParamYN = "Updated parameter in user specified parameter list"
        elif parameter2update in defaultParameters:
            value2write = self.valueEntry.get()
            userSpecifiedParameters[parameter2update] = value2write
            paramInfo = parameter2update + ': ' + value2write
            newParamYN = "Added new parameter in user specified parameter list."
        else:
            paramInfo = '"' + parameter2update + '" does not exist.'
            newParamYN = "-"

        self.resultTitle.set("Parameter Information")
        self.result.set(paramInfo)
        self.notesTitle.set("Notes")
        self.notes.set(newParamYN)

        self.resultLabel = Label(self.frame, textvariable = self.resultTitle)
        self.resultLabel.grid(row = 1, column = 5, padx = 10, columnspan = 2)
        
        self.resultValue = Label(self.frame, textvariable = self.result)
        self.resultValue.grid(row = 2, column = 5, padx = 10, columnspan = 2)

        self.notesLabel = Label(self.frame, textvariable = self.notesTitle)
        self.notesLabel.grid(row = 1, column = 7, padx = 10)

        self.notesValue = Label(self.frame, textvariable = self.notes)
        self.notesValue.grid(row = 2, column = 7, padx = 10)

    def listDefaultParameters(self):
        listParameters(defaultParameters)

    def listSpecifiedParameters(self):
        listParameters(userSpecifiedParameters)

    def promptSaveParameter(self):

        self.directoryLabel = Label(self.frame, text = "Directory: ")
        self.directoryLabel.grid(row = 3, column = 5, padx = 20, pady = 10)
        
        self.newFileDirectoryEntry = Entry(self.frame, width = 25)
        self.newFileDirectoryEntry.grid(row = 3, column = 6, padx = 20, pady = 10)

        self.nameLabel = Label(self.frame, text = "File Name: ")
        self.nameLabel.grid(row = 4, column = 5, padx = 20, pady = 10)

        self.newFileDirectoryEntry.delete(0, END)
        self.newFileDirectoryEntry.insert(0, newFileDirectory)

        self.newFileNameEntry = Entry(self.frame, width = 25)
        self.newFileNameEntry.grid(row = 4, column = 6, padx = 20, pady = 10)

        self.newFileNameEntry.delete(0, END)
        self.newFileNameEntry.insert(0, newFileName)

        self.saveButton = Button(self.frame, text = "Save to file", command = self.saveParameter)
        self.saveButton.grid(row = 5, column  = 6, padx = 20, pady = 10, sticky = E)

    def saveParameter(self):

        self.newFilePath = self.newFileDirectoryEntry.get() + self.newFileNameEntry.get()
        
        writeParamFile(self.newFilePath, userSpecifiedParameters)
        


        

       
        
        
  
def main():
    root = Tk()
    root.wm_title("Duchamp Parameters")
  
    interface = mainInterface(root)

  
    root.mainloop()
    root.destroy()

#-------------------------------------------------------------------------------     
    
##def main():
##    editParameter()
##    checkParameter()
    
if __name__ == '__main__':
    main()
